@@ -20,6 +20,7 @@ from .astutil import Comment
 from .astutil import Symbol
 from .astutil import Builtin
 from .astutil import Static
+from .astutil import TokenRef
 
 from .codegen import TemplateCodeGenerator
 from .codegen import template
@@ -107,15 +108,13 @@ def store_rcontext(name):
     return subscript(name, load("rcontext"), ast.Store())
 
 
-def set_error(exception):
-    return template(
-        "rcontext.setdefault('__error__', [])."
-        "append(__token + (exc, ))",
-        exc=exception,
-    )
-
-
 def set_token(stmts, token):
+    pos = getattr(token, "pos", 0)
+    body = template("__token = pos", pos=TokenRef(pos, len(token)))
+    return body + stmts
+
+
+def eval_token(token):
     try:
         line, column = token.location
         filename = token.filename
@@ -125,44 +124,43 @@ def set_token(stmts, token):
 
     string = safe_native(token)
 
-    body = template(
-        "__token = string, line, col, src",
+    return template(
+        "(string, line, col)",
         string=ast.Str(s=string),
         line=ast.Num(n=line),
         col=ast.Num(n=column),
-        src=ast.Str(s=filename),
+        mode="eval"
     )
 
-    return body + stmts
+
+emit_node = template(is_func=True, func_args=('node',), source=r"""
+    __append(node)""")
 
 
-@template
-def emit_node(node):  # pragma: no cover
-    __append(node)
-
-
-@template
-def emit_node_if_non_trivial(node):  # pragma: no cover
+emit_node_if_non_trivial = template(is_func=True, func_args=('node',),
+                                    source=r"""
     if node is not None:
         __append(node)
+""")
 
 
-@template
-def emit_bool(target, s, default_marker=None,
-                 default=None):  # pragma: no cover
+emit_bool = template(is_func=True,
+                     func_args=('target', 's', 'default_marker', 'default'),
+                     func_defaults=(None, None), source=r"""
     if target is default_marker:
         target = default
     elif target:
         target = s
     else:
-        target = None
+        target = None""")
 
 
-@template
-def emit_convert(
-    target, encoded=byte_string, str=unicode_string,
-    long=long, type=type,
-    default_marker=None, default=None):  # pragma: no cover
+emit_convert = template(is_func=True,
+                        func_args=('target', 'encoded', 'str', 'long', 'type',
+                                   'default_marker', 'default'),
+                        func_defaults=(byte_string, unicode_string, long, type,
+                                       None),
+                        source=r"""
     if target is None:
         pass
     elif target is default_marker:
@@ -181,13 +179,14 @@ def emit_convert(
                 __converted = convert(target)
                 target = str(target) if target is __converted else __converted
             else:
-                target = target()
+                target = target()""")
 
 
-@template
-def emit_func_convert(
-    func, encoded=byte_string, str=unicode_string,
-    long=long, type=type):  # pragma: no cover
+emit_func_convert = template(is_func=True,
+                             func_args=('func', 'encoded', 'str','long','type'),
+                             func_defaults=(byte_string, unicode_string, long,
+                                            type),
+                             source=r"""
     def func(target):
         if target is None:
             return
@@ -209,20 +208,24 @@ def emit_func_convert(
             else:
                 target = target()
 
-        return target
+        return target""")
 
 
-@template
-def emit_translate(target, msgid, target_language, default=None):  \
-    # pragma: no cover
-    target = translate(msgid, default=default, domain=__i18n_domain, context=__i18n_context, target_language=target_language)
+emit_translate = template(is_func=True,
+                          func_args=('target', 'msgid', 'target_language',
+                          'default'),
+                          func_defaults=(None,),
+                          source=r"""
+    target = translate(msgid, default=default, domain=__i18n_domain, 
+                       context=__i18n_context, 
+                       target_language=target_language)""")
 
 
-@template
-def emit_func_convert_and_escape(
-    func, str=unicode_string, long=long,
-    type=type, encoded=byte_string):  # pragma: no cover
-
+emit_func_convert_and_escape = template(
+    is_func=True,
+    func_args=('func', 'str', 'long', 'type', 'encoded'),
+    func_defaults=(unicode_string, long, type, byte_string,),
+    source=r"""
     def func(target, quote, quote_entity, default, default_marker):
         if target is None:
             return
@@ -264,7 +267,7 @@ def emit_func_convert_and_escape(
                         if quote is not None and quote in target:
                             target = target.replace(quote, quote_entity)
 
-        return target
+        return target""")
 
 
 class Interpolator(object):
@@ -638,8 +641,9 @@ class ExpressionEvaluator(object):
             module = Module("evaluate", Context(assignment))
 
             compiler = Compiler(
-                self._engine, module, ('econtext', 'rcontext') + self._names
-                )
+                self._engine, module, "<string>", string,
+                ('econtext', 'rcontext') + self._names
+            )
 
             env = {}
             exec(compiler.code, env)
@@ -760,13 +764,9 @@ class ExpressionTransform(object):
 
             stmts = template(
                 "__exc = loads(p)", loads=self.loads_symbol, p=ast.Str(s=p)
-                )
+            )
 
-            token = Token(exc.token, exc.offset, filename=exc.filename)
-
-            stmts = set_token(stmts, token)
-            stmts += set_error(load("__exc"))
-            stmts += [ast.Raise(exc=load("__exc"))]
+            stmts += set_token([ast.Raise(exc=load("__exc"))], exc.token)
 
         # Apply visitor to each statement
         for stmt in stmts:
@@ -907,7 +907,8 @@ class Compiler(object):
 
     global_builtins = set(builtins.__dict__)
 
-    def __init__(self, engine_factory, node, builtins={}, strict=True):
+    def __init__(self, engine_factory, node, filename, source,
+                 builtins={}, strict=True):
         self._scopes = [set()]
         self._expression_cache = {}
         self._translations = []
@@ -944,14 +945,27 @@ class Compiler(object):
             module = ast.Module([])
             module.body += self.visit(node)
             ast.fix_missing_locations(module)
-            generator = TemplateCodeGenerator(module)
+            prelude = "__filename = %r" % filename
+            generator = TemplateCodeGenerator(module, source)
+            tokens = [
+                Token(source[pos:pos + length], pos, source)
+                for pos, length in generator.tokens
+            ]
+            token_map_def = "__tokens = {" + ", ".join("%d: %r" % (
+                token.pos,
+                (token, ) + token.location
+            ) for token in tokens) + "}"
         finally:
             if backup is not None:
                 node_annotations.clear()
                 node_annotations.update(backup)
                 self.lock.release()
 
-        self.code = generator.code
+        self.code = "\n".join((
+            prelude,
+            token_map_def,
+            generator.code
+        ))
 
     def visit(self, node):
         if node is None:
@@ -1041,7 +1055,7 @@ class Compiler(object):
         # Initialization
         body += template("__append = __stream.append")
         body += template("__re_amp = g_re_amp")
-        body += template("__token = ()")
+        body += template("__token = None")
         body += template("__re_needs_escape = g_re_needs_escape")
 
         body += emit_func_convert("__convert")
@@ -1067,9 +1081,17 @@ class Compiler(object):
                 "except: NAME = None",
                 KEY=ast.Str(s=name), NAME=store(name))
 
-        exc_handler = set_error(template(
+        exc = template(
             "exc_info()[1]", exc_info=Symbol(sys.exc_info), mode="eval"
-        )) + template("raise")
+        )
+
+        exc_handler = template(
+            "if pos is not None: rcontext.setdefault('__error__', [])."
+            "append(token + (__filename, exc, ))",
+            exc=exc,
+            token=template("__tokens[pos]", pos="__token", mode="eval"),
+            pos="__token"
+        ) + template("raise")
 
         # Wrap visited nodes in try-except error handler.
         body += [
@@ -1126,7 +1148,7 @@ class Compiler(object):
         self._leave_assignment((node.name, ))
 
         error_assignment = template(
-            "econtext[key] = cls(__exc, __token[1:3])",
+            "econtext[key] = cls(__exc, __tokens[__token][1:3])",
             cls=ErrorInfo,
             key=ast.Str(s=node.name),
             )
